#!/bin/bash
#Takes as input a psf file, dcd file, output directory
#Produces x3dna analysis in the output directory with extension .x3
#INPUT ./x3dna.sh <PSF FILE> <DCD FILE> <OUTPUT DIRECTORY>

# remove=1 removes intermediate files after concatenation (saves space)
# remove=0 does not remove files (helps with debugging)
remove=1
#chunksize=100 concatenates pdb files into 100 file chunks.
#Too large of a number will hurt performance
chunksize=100

psf=$1
dcd=$2
outdir=$3
# Name of output stripped dcd
strdcd=$outdir/`basename ${dcd%.*}`.str.dcd
# name of pdb file used as input for find_pair
pdb=$outdir/`basename ${psf%.*}`
#name of pdb file without any suffix
pdbase=`echo $pdb|rev|cut -d '/' -f1|rev`
echo "$strdcd $pdb $pdbase"

#  CREATES A STRIPPED DCD FILE (JUST DNA)
mkdir $outdir
vmd -dispdev text -e strip.tcl -args $psf $dcd $outdir
#catdcd -o $strdcd -i $pdb.ind $dcd
if [ $remove -gt 0 ]
 then
  rm $pdb.ind
fi

#  CREATES A FILE THAT LISTS BASEPAIRS THAT SHOULD EXIST -- NEEDED BY X3DNA
find_pair $pdb.str.pdb $pdb.bps 

#  WRITES OUT PDB FILES FOR EVERY FRAME IN THE DCD -- USING THE CREATED STRIPPED DCD
vmd -dispdev text -e writepdbs.tcl -args $pdb.str.psf $strdcd $outdir/dna. > out.txt

#  CONCATENATES PDB FILES INTO COMPOSITE PDB FILES CONTAINING chunksize FRAMES PER FILE
numfiles=`ls $outdir/dna.*.pdb|wc -w`
numchunks=$(( $numfiles / $chunksize ))
leftover=$(( $numfiles - $numchunks * $chunksize ))

filelist=`ls $outdir/dna.*.pdb`
for I in `seq $numchunks`
do
  beg=$(( ( $I - 1 ) * $chunksize + 1 )) 
  end=$((  $I * $chunksize )) 
  echo "$beg $end"
  cat `echo $filelist|cut -d' ' -f$beg-$end` >$outdir/combined.$I.pdb 
  if [ $remove -gt 0 ]
   then
    rm `echo $filelist|cut -d' ' -f$beg-$end`
  fi
done
if [ $leftover -gt 0 ]
 then 
  beg=$(( $numchunks * $chunksize + 1 ))
  end=$numfiles
  echo "$beg $end" 
  numchunks=$(( $numchunks + 1 ))
  cat `echo $filelist|cut -d' ' -f$beg-$end` >$outdir/combined.$numchunks.pdb
  if [ $remove -gt 0 ]
   then
    rm `echo $filelist|cut -d' ' -f$beg-$end`
  fi
fi

#  FORMATTING CONCATENATED PDB FILES
for I in `seq $numchunks`
do
  awk '{if ($1=="CRYST1") {n = n+1;print "MODEL",n} else print}' $outdir/combined.$I.pdb|sed -e  's/END/ENDMDL/g' >$outdir/total.$I.pdb
  if [ $remove -gt 0 ]
   then
    rm $outdir/combined.$I.pdb
  fi
done

# X3DNA ANALYSIS
for I in `seq $numchunks`
do
  x3dna_ensemble analyze -b $pdb.bps -e $outdir/total.$I.pdb -o $outdir/dna.$I.x3
  if [ $remove -gt 0 ]
   then
    rm $outdir/total.$I.pdb
  fi
done

./justpy.sh $outdir

